<?php

namespace Crm\Brand\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Crm\Brand\Events\BrandWorkflow as BrandWorkflowEvent;
use Crm\Brand\Notifications\BrandWorkflow as BrandWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class BrandWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(BrandWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $brand is submitted.
     *
     * @param BrandWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(BrandWorkflowEvent $event)
    {
        $client = $event->$brand->client;
        Notification::send($client, new BrandWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $brand is published.
     *
     * @param BrandWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(BrandWorkflowEvent $event)
    {
        $client = $event->$brand->client;
        Notification::send($client, new BrandWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $brand is approved.
     *
     * @param BrandWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(BrandWorkflowEvent $event)
    {
        $client = $event->$brand->client;
        Notification::send($client, new BrandWorkflowNotification($event));
    }

    /**
     * Handles the $brand workflow event as a listener.
     *
     * @param BrandWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(BrandWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
